// StatusBarCtrl.cpp cCve[Vt@C
#include "StatusBarCtrl.h"
#include "DeviceContext.h"              // wgc::DeviceContext
#include "wgfunc.h"                     // wgc::wgfLoadString()
#include <assert.h>                     // assert()


_WGC_BEGIN                              // namespace wgc {

// RXgN^
StatusBarCtrl::StatusBarCtrl(void)
{
}

// fXgN^
StatusBarCtrl::~StatusBarCtrl(void)
{
}


// Xe[^Xo[̍쐬
BOOL StatusBarCtrl::Create(
	Window *pParent,
	LPCTSTR lpszWindowName,
	const DWORD dwStyle          /* = WS_CHILD | WS_VISIBLE */,
	const DWORD dwCtlStyle       /* = CCS_BOTTOM */,
	const DWORD dwStatusBarStyle /* = SBARS_SIZEGRIP */,
	const DWORD dwExStyle        /* = 0 */,
	const UINT  nID              /* = IDW_STD_STATUSBAR */)
{
	assert(pParent != NULL);

	return Control::Create(
		STATUSCLASSNAME, lpszWindowName,
		dwStyle, dwCtlStyle | dwStatusBarStyle,
		dwExStyle,
		pParent,
		nID);
}


// CWP[^ݒ
BOOL StatusBarCtrl::SetIndicators(HINSTANCE hResource, const UINT nIDArray[], const int nCount)
{
	m_pane_width.resize(nCount);

	// e̕擾
	for(int i = 0; i < nCount; i++)
	{
		tstring_t tstr;
		wgfLoadString(hResource, nIDArray[i], tstr);

		SIZE size;
		DeviceContext_Client dc(m_hWnd);
		dc.GetTextExtent(tstr, size);

		m_pane_width[i] = size.cx;
	}

	return UpdatePanePosition();
}


////////////////////////////////////////////////////////////////////////////////
// 

// Vv[h
BOOL StatusBarCtrl::SetSimple(const BOOL bSimple /* = TRUE */)
{
	const LRESULT lResult = SendMessage(
		SB_SIMPLE,
		bSimple);
	return static_cast<BOOL>(lResult);
}

// E擾iSvf𓯎Ɂj
BOOL StatusBarCtrl::GetBorders(int *pBorders) const
{
	assert(pBorders != NULL);
	const LRESULT lResult = SendMessage(
		SB_GETBORDERS,
		0,
		reinterpret_cast<LPARAM>(pBorders));
	return static_cast<BOOL>(lResult);
}

// E擾ivfj
BOOL StatusBarCtrl::GetBorders(int *pHorz, int *pVert, int *nSpacing) const
{
	int arBorders[3];
	if(!GetBorders(arBorders))
	{
		return FALSE;
	}

	if(pHorz    != NULL) { *pHorz    = arBorders[0]; }
	if(pVert    != NULL) { *pVert    = arBorders[1]; }
	if(nSpacing != NULL) { *nSpacing = arBorders[2]; }
	return TRUE;
}

// yC쐬
BOOL StatusBarCtrl::SetParts(const int nParts, int *pWidths)
{
	assert(pWidths != NULL);
	const LRESULT lResult = SendMessage(
		SB_SETPARTS,
		nParts,
		reinterpret_cast<LPARAM>(pWidths));
	return static_cast<BOOL>(lResult);
}

// yC擾
BOOL StatusBarCtrl::GetParts(const int nParts, int *pWidths) const
{
	assert(pWidths != NULL);
	const LRESULT lResult = SendMessage(
		SB_GETPARTS,
		nParts,
		reinterpret_cast<LPARAM>(pWidths));
	return static_cast<BOOL>(lResult);
}

// eLXgo
BOOL StatusBarCtrl::SetText(const int iPart, const int uType, const tstring_t &text)
{
	const LRESULT lResult = SendMessage(
		SB_SETTEXT,
		iPart | uType,
		reinterpret_cast<LPARAM>(text.c_str()));
	return static_cast<BOOL>(lResult);
}


////////////////////////////////////////////////////////////
// bZ[Wnh
void StatusBarCtrl::OnSize(const UINT nType, const int cx, const int cy)
{
	Control::OnSize(nType, cx, cy);
	UpdatePanePosition();
}


////////////////////////////////////////////////////////////
// private

// ̗̈vZ
void StatusBarCtrl::CalcInsideRect(RECT &rect) const
{
	GetClientRect(rect);

	// ő\łȂ΃TCYύẌl
	if(!::IsZoomed(GetParent()))
	{
		// TCYύẌ́AقڃNCAg̈̍Ɠ
		rect.right -= (rect.bottom - rect.top);
	}
}

// eyC̈ʒuXV
BOOL StatusBarCtrl::UpdatePanePosition(void)
{
	// ACh\̈{CWP[^
	const size_t count = m_pane_width.size();
	intarray_t right_pos(count + 1);

	// {block}
	// ԉẼyC̍W
	{
		RECT rect;
		CalcInsideRect(rect);
		right_pos[count] = rect.right;
	}

	// ̑̃yC̍W
	for(size_t j = count; j > 0; j--)
	{
		right_pos[j - 1] = right_pos[j] - m_pane_width[j - 1];
	}

	return SetParts(
		static_cast<int>(right_pos.size()),
		&right_pos[0]);
}

_WGC_END                                // }
